// import axios from 'axios';
import Koa from 'koa'
import log4js from './utils/log4js';
import textComplRouter from './router/text-compl-router';
import chatComplRouter from './router/chat-completion-router';
import ftRouter from './router/fine-tuning-router';
import userRouter from './router/user-router';
import wisperTransRouter from './router/wisper-transcription-router';
import fs = require('fs');

import { txt2jsonl } from './utils/compile/node_txt2jsonl';
import { xlsx2jsonl } from './utils/compile/node_xlsx2jsonl';

const Router = require('koa-router'); // 引入koa-router
const koaBody = require('koa-body');
// const Pug = require('koa-pug')
const views = require('koa-views')
const path = require('path')

const app = new Koa()
app.use(views(__dirname + '/views', {
  extension: 'pug'
  // map: {
  //   html: 'pug'
  // }
}))

const staticServer = require('koa-static')
app.use(staticServer(path.join(__dirname, '../public')))

function getUploadFileExt(name: string) {
  let extPcs = name.split('.');
  return extPcs[extPcs.length - 1];
}
function getUploadFileName(name: string) {
  let namePcs = name.split('/');
  // console.log('dddddddddddddddddd', namePcs)
  if (namePcs.length > 1) {
    return namePcs[namePcs.length - 1];
  }
  else {
    return name;
  }
}

function checkDirExist(dPath: string) {
  if (!fs.existsSync(dPath)) {
    fs.mkdirSync(dPath);
  }
}
function getUploadDirName() {
  const date = new Date();
  let month = date.getMonth() + 1;
  let monthStr = month.toString().length > 1 ? String(month) : `0${month}`;
  let dtDay = date.getDate()
  let dtDayStr = dtDay.toString().length > 1 ? String(dtDay) : `0${dtDay}`
  const dir = `${date.getFullYear()}${monthStr}${dtDayStr}`;
  return dir;
}
app.use(
  koaBody({
    multipart: true,
    strict: false, // 设为false
    // encoding: 'gzip',
    formidable: {
      uploadDir: path.join(__dirname, '../uploads/'), //必要
      keepExtensions: true,
      maxFileSize: 10 * 1024 * 1024,
      onFileBegin: (name: string, file: any) => {
        // // 获取文件后缀
        // const ext = getUploadFileExt(file.originalFilename);

        // 最终要保存到的文件夹目录
        const dirPar = path.join(__dirname, `../uploads`)
        checkDirExist(dirPar);

        // 检查文件夹是否存在如果不存在则新建文件夹
        const dir = path.join(__dirname, `../uploads/${getUploadDirName()}`);
        checkDirExist(dir);

        // 重新覆盖 file.path 属性
        const fileName = file.originalFilename;
        file.filepath = `${dir}/${fileName}`; //必要
      },
      onError: (err: any) => {
        console.log(err);
      }
    }
  })
)

const onerror = require("koa-onerror")
onerror(app)

// logger (需要在router绑定之前声明)
app.use(async (ctx: any, next: any) => {
  // console.log('w1')

  const start = Date.now()
  await next()
  // console.log('w2')
  const ms = Date.now() - start

  log4js.resLogger(ctx, ms)
  // console.log('w3')
})

// error-handling
app.on("error", (err: any, ctx: any) => {
  log4js.errLogger(ctx, err)

  if (err) {
    console.log("应用错误：", err)
  }
  if (ctx) {
    console.log("错误信息：", JSON.stringify(ctx.onerror))
  }
})

const router = new Router()
app.use(router.allowedMethods())
app.use(router.routes())
router.use("/tc", textComplRouter.routes(), textComplRouter.allowedMethods())
router.use("/cc", chatComplRouter.routes(), chatComplRouter.allowedMethods())
router.use("/v2t", wisperTransRouter.routes(), wisperTransRouter.allowedMethods())
router.use("/ft", ftRouter.routes(), ftRouter.allowedMethods())
router.use("/usr", userRouter.routes(), userRouter.allowedMethods())

// app.listen(7001, () => {
//   console.log('应用已经启动, http://localhost:7001');
// })

app.listen(17001, () => {
  console.log('应用已经启动, http://localhost:17001');
})


// txt2jsonl()
// xlsx2jsonl()
