import fetch, { Response } from 'node-fetch';
import { CONST_GPT35_MODEL, CONST_DEFAULT_CHATCOMPLETION_PARAM, CONST_GPT30_MODEL, CONST_OPENAI_COM_KEY } from "../config";
import { ChatCompletionMessage, ChatCompletionParams } from "../schemas/chat-completion";
import { ICommonResult } from "../utils/http";

const Router = require('koa-router'); // 引入koa-router
const chatComplRouter = new Router()
chatComplRouter.get('/demo', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index-chat.pug', {
    // title: "单句问答(OpenAi-davinci)对照实验",
    openGPT35Name: CONST_GPT35_MODEL,
    openGPT30Name: CONST_GPT30_MODEL,
    // openFTNameRp4: CONST_DAVINCI_FINE_TUNNING_REPEAT4,
  });
})

chatComplRouter.get('/demo2', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index-chat2.pug', {
    // title: "单句问答(OpenAi-davinci)对照实验",
    openGPT35Name: CONST_GPT35_MODEL,
    openGPT30Name: CONST_GPT30_MODEL,
    // openFTNameRp4: CONST_DAVINCI_FINE_TUNNING_REPEAT4,
  });
})

chatComplRouter.get('/demo1', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index-chat1.pug', {
    // title: "单句问答(OpenAi-davinci)对照实验",
    openGPT35Name: CONST_GPT35_MODEL,
    // openGPT30Name: CONST_GPT30_MODEL,
    // openFTNameRp4: CONST_DAVINCI_FINE_TUNNING_REPEAT4,
  });
})

chatComplRouter.post('/completionByModelByComKey', async (ctx: {
  request: {
    body: {
      messages: ChatCompletionMessage[],
      userId: string,
      model: string,
    }
  },
  body: ICommonResult<any>
}, next: Function) => {
  console.log('chatComplRouter.completionByModelByComKey start:')
  let messages = []
  let user = ''
  let model = ''
  console.log("ctx.request.body=> type:", typeof ctx.request.body, JSON.stringify(ctx.request.body))
  if (typeof ctx.request.body === "string") {
    let json = JSON.parse(ctx.request.body)
    messages = json.messages
    user = json.userId
    model = json.model
  }
  else {
    messages = ctx.request.body.messages
    user = ctx.request.body.userId
    model = ctx.request.body.model
  }

  let resp: Response
  let completionParams = JSON.parse(JSON.stringify(CONST_DEFAULT_CHATCOMPLETION_PARAM)) as ChatCompletionParams
  completionParams.user = user
  if (model) completionParams.model = model
  completionParams.messages = messages
  // completionParams.max_tokens = 2000
  console.log('使用公司key...', {
    method: "post",
    headers: {
      'Authorization': 'Bearer ' + CONST_OPENAI_COM_KEY,
      'Content-Type': 'application/json'
    },
    body: JSON.stringify(completionParams)
  })
  try {
    resp = await fetch('https://api.openai.com/v1/chat/completions', {
      method: "post",
      headers: {
        'Authorization': 'Bearer ' + CONST_OPENAI_COM_KEY,
        'Content-Type': 'application/json'
      },
      body: JSON.stringify(completionParams)
    })
  } catch (ex: any) {
    ctx.body = {
      Result: 0,
      code: 10510,
      Reason: "fetch err:" + ex.message
    };
    return
  }

  const json = await resp.json()
  if (json && json.error) {
    ctx.body = {
      Result: 0,
      code: 500,
      data: json
    }
    return
  }
  ctx.body = {
    Result: 1,
    code: 200,
    data: json
  }
})

export default chatComplRouter
