
import { readFileSync, appendFile, appendFileSync, writeFileSync, writeFile, readFile } from "fs";
import path, { resolve } from 'path'
import { CONST_COLPLETION_END, CONST_COLPLETION_START, CONST_FINE_TUNNING_ENGINE, CONST_OPENAI_KEY } from "../config";
import { FineTuningCreateParams } from "../schemas/fine-tuning";
import { ICommonResult } from "../utils/http";


const Router = require('koa-router'); // 引入koa-router
const ftRouter = new Router()
const _nextJsonLPathName = 'cases_for_coach2_prepared_next.jsonl'
const _nextJsonLFulPathName = path.resolve(__dirname, '../../public/' + _nextJsonLPathName)

ftRouter.get('/ftdemo', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('fine-tuning.pug', {
    title: "openai fine tuning test",
    jsonlName: 'cases_for_coach_prepared_next.jsonl',
    list: []
  });
})

// #region next.jsonl管理
/**
 * 为next.jsonl增加内容(训练助手)
 */
ftRouter.post('/jmg/addNextJsonLItem', async (ctx: {
  request: {
    body: {
      prompt: string,
      completion: string
    }
  },
  body: ICommonResult<boolean>
}, next: Function) => {
  // 格式化检查之后(存储到数据库，定时生成)更新到cases_for_coach_next_prepared.jsonl文件之中
  const jsonStr = `\n{"prompt":"${ctx.request.body.prompt}","completion":${CONST_COLPLETION_START}"${ctx.request.body.completion}${CONST_COLPLETION_END}"}`
  const rlt = await new Promise<boolean>((resolve) => {
    appendFile(_nextJsonLPathName, jsonStr, { encoding: 'utf-8' }, (err) => {
      if (err) {
        resolve(false)
      }
      else {
        resolve(true)
      }
    })
  })

  if (rlt) {
    ctx.body = {
      Result: 1,
      code: 200,
      data: true
    }
  }
  else {
    ctx.body = {
      Result: 0,
      code: 500,
      data: false
    }
  }
})

/**
 * 获取全部的next.jsonl内容(admin)
 */
ftRouter.get('/jmg/getNextJsonLAll', async (ctx: {
  request: {
    body: {}
  },
  body: ICommonResult<string>
}, next: Function) => {
  // 格式化检查之后(存储到数据库，定时生成)更新到cases_for_coach_next_prepared.jsonl文件之中
  let jsonlListStr = ""
  let readSuccess = false
  try {
    jsonlListStr = await readFileSync(_nextJsonLFulPathName, { encoding: 'utf-8' }) // , flag: 'r'
    readSuccess = true
  }
  catch (e: any) { }
  if (readSuccess) {
    ctx.body = {
      Result: 1,
      code: 200,
      data: jsonlListStr
    }
  }
  else {
    ctx.body = {
      Result: 0,
      code: 500,
      Reason: "read file failed"
    }
  }
})

/**
 * 全局更新next.jsonl内容(admin)
 */
ftRouter.get('/jmg/updateNextJsonL', async (ctx: {
  request: {
    body: {
      jsonlListStr: string
    }
  },
  body: ICommonResult<boolean>
}, next: Function) => {

  const { jsonlListStr } = ctx.request.body
  // 1.格式化检查之后(存储到数据库，定时生成)更新到cases_for_coach_next_prepared.jsonl文件之中

  // 2.写入
  const writeRlt = await new Promise<boolean>(resolve => {
    writeFile(_nextJsonLFulPathName, jsonlListStr, 'utf-8', (err) => {
      if (err) {
        resolve(false)
      }
      else {
        resolve(true)
      }
    })
  })

  if (writeRlt) {
    ctx.body = {
      Result: 1,
      code: 200,
      data: true
    }
  }
  else {
    ctx.body = {
      Result: 0,
      code: 500,
      Reason: "write failed"
    }
  }
})
// #endregion

// #region job管理
// /**
//  * 拷贝next.jsonl到next_prepare.jsonl
//  * 提交next.jsonl内容进行训练
//  * (admin)
//  */
// ftRouter.post('/submitJsonLForFtJob', async (ctx: {
//   request: {
//     body: {

//     }
//   },
//   body: ICommonResult<string>
// }, next: Function) => {
//   // 格式化检查之后(存储到数据库，定时生成)更新到cases_for_coach_next_prepared.jsonl文件之中

//   // 方法1 上传
//   // export OPENAI_API_KEY="xxxxx" && openai api fine_tunes.create -t ./open-ai-server/public/cases_for_coach2_prepared.jsonl -m davinci
// })

// /**
//  * 查看训练进度
//  * (admin)
//  */
// ftRouter.get('/checkIsJobFinished', async (ctx: {
//   request: {
//     body: {
//       jobId: string
//     }
//   },
//   body: ICommonResult<boolean>
// }, next: Function) => {
//   // 格式化检查之后(存储到数据库，定时生成)更新到cases_for_coach_next_prepared.jsonl文件之中
//   // https://api.openai.com/v1/fine-tunes/ft-OMgNO9BNhLcZ3VvwRKYlcmiR

// })

// /**
//  * 继续未完成的训练生成ft (使用train_file创建ft -> 重复)
//  * (admin)
//  */
// ftRouter.post('/continueJob', async (ctx: {
//   request: {
//     body: {
//       jobId: string
//     }
//   },
//   body: ICommonResult<boolean>
// }, next: Function) => {

// })
// #endregion

// #region 训练文件管理
/**
 * 上传jsonl文件，为后续的训练做准备
 */
ftRouter.post('/fmg/submitJSONLFile', async (ctx: {
  request: {
    body: {}
  },
  body: ICommonResult<string>
}, next: Function) => {
  // 1.上传
  const data = new FormData()
  data.append("purpose", "fine-tune")
  const binary = new Uint8Array([97, 98, 99])
  const file = new File([binary], _nextJsonLFulPathName, { type: 'text/plain' })
  data.append("file", file, "")
  const res = await fetch('https://api.openai.com/v1/files', {
    method: "POST",
    headers: {
      "Content-Type": "Content-Type: application/x-www-form-urlencoded",
      "Authorization": "Bearer " + CONST_OPENAI_KEY
    },
    body: data
  }).then(json => json.json())
  if (res && res.id) {
    // 2.写入
    const _newPath = _nextJsonLFulPathName.replace('.jsonl', res.id + '.jsonl')
    const writeRlt = await new Promise<boolean>(resolve => {
      writeFile(_newPath, _newPath, 'utf-8', (err) => {
        if (err) {
          resolve(false)
        }
        else {
          resolve(true)
        }
      })
    })
    if (writeRlt) {
      ctx.body = {
        Result: 1,
        code: 200,
        data: res.id
      }
      return
    }
  }
  ctx.body = {
    Result: 0,
    code: 500,
    Reason: res ? JSON.stringify(res) : 'unknown'
  }
})

/**
 * 查看文件状态
 */
ftRouter.post('/fmg/getTrainFileStatus', async (ctx: {
  request: {
    body: {
      trainFileId: string
    }
  },
  body: ICommonResult<string>
}, next: Function) => {
  // get https://api.openai.com/v1/files/file-qZugmzvPD2zLstIgR2odEF3r\
  const { trainFileId } = ctx.request.body
  const res = await fetch('https://api.openai.com/v1/files/' + trainFileId, {
    method: "GET",
    headers: {
      "Authorization": "Bearer " + CONST_OPENAI_KEY
    }
  }).then(res => res.json())
  if (res && res.message) {
    ctx.body = {
      Result: 0,
      code: 500,
      Reason: res ? res.message : "no result"
    }
  }
  else {
    ctx.body = {
      Result: 1,
      code: 200,
      data: res
    }
  }
})

/**
 * 使用训练文件指定引擎创建模型
 * */
ftRouter.post('/fmg/trainByFile', async (ctx: {
  request: {
    body: {
      trainFileId: string
    }
  },
  body: ICommonResult<string>
}, next: Function) => {
  const  { trainFileId } = ctx.request.body
  const res = await fetch("https://api.openai.com/v1/fine-tunes", {
    method: "POST",
    headers: {
      "Authorization": "Bearer " + CONST_OPENAI_KEY
    },
    body: JSON.stringify({
      training_file: trainFileId,
      model: CONST_FINE_TUNNING_ENGINE
    } as FineTuningCreateParams)
  }).then(json=>json.json())

  // 主要看 res.status === pending ? 还是 succeed
  if (res && res.id) {

  }
  else {

  }
})

/**
 * 获取模型训练状态
 * */
ftRouter.post('/fmg/getTrainFileStatus', async (ctx: {
  request: {
    body: {
      ftId: string
    }
  },
  body: ICommonResult<any>
}, next: Function) => {
  const { ftId } = ctx.request.body
  const res = await fetch('https://api.openai.com/v1/fine-tunes/' + ftId, {
    method: "GET",
    headers: {
      "Authorization": "Bearer " + CONST_OPENAI_KEY
    }
  }).then(json => json.json())

  if (res && res.status === "succeeded") {
    ctx.body = {
      Result: 1,
      code: 200,
      data: res
    }
  }
  else {
    ctx.body = {
      Result: 0,
      code: 500,
      data: res
    }
  }
})
// #endregion

export default ftRouter
