import fetch, { Response } from 'node-fetch';
import { CONST_ADA_FINE_TUNNING, CONST_ADA_FINE_TUNNING_REPEAT2, CONST_ADA_FINE_TUNNING_REPEAT4, CONST_BABBAGE_FINE_TUNNING, CONST_BABBAGE_FINE_TUNNING_REPEAT2, CONST_BABBAGE_FINE_TUNNING_REPEAT4, CONST_COLPLETION_END, CONST_CURIE_FINE_TUNNING, CONST_CURIE_FINE_TUNNING_REPEAT2, CONST_CURIE_FINE_TUNNING_REPEAT4, CONST_DAVINCI_FINE_TUNNING, CONST_DAVINCI_FINE_TUNNING_REPEAT2, CONST_DAVINCI_FINE_TUNNING_REPEAT4, CONST_DAVINCI_USERINFO_FINE_TUNNING, CONST_DAVINCI_USERINFO_FINE_TUNNING_RP2, CONST_DEFAULT_COMPLETION_PARAM, CONST_OPENAI_COM_KEY, CONST_OPENAI_KEY, CONST_OPENAI_MAX_TOKEN } from '../config';
import { CompletionParams } from '../schemas/text-completion';
import { xlsx2arr } from '../utils/compile/node_xlsx2jsonl';
import { ICommonResult, user_agents } from "../utils/http";
// import log4js from "../utils/log4js"
// const compose = require("koa-compose")
const Router = require('koa-router'); // 引入koa-router
const textComplRouter = new Router()
let demoUserInfos: { userId: string, desc: string }[] | undefined

textComplRouter.get('/demo', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index.pug', {
    // title: "单句问答(OpenAi-davinci)对照实验",
    // openFTName: CONST_DAVINCI_FINE_TUNNING,
    // openFTNameRp2: CONST_DAVINCI_FINE_TUNNING_REPEAT2,
    // openFTNameRp4: CONST_DAVINCI_FINE_TUNNING_REPEAT4,
  });
})
/**
 * 默认获取一个页面执行/getImgUrlFromMmsUrl解析操作，(注意mms彩信，好像只有一周的有效期)
 */
textComplRouter.get('/demo1', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index1.pug', {
    // title: "单句问答(OpenAi-davinci)对照实验",
    openFTName: CONST_DAVINCI_FINE_TUNNING,
    openFTNameRp2: CONST_DAVINCI_FINE_TUNNING_REPEAT2,
    openFTNameRp4: CONST_DAVINCI_FINE_TUNNING_REPEAT4,
  });
})

textComplRouter.get('/dlg', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('dlg.pug', {
    // title: "会话式问答",
  });
})

textComplRouter.get('/demo2', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index2.pug', {
    // title: "单句问答(OpenAi-davinci/curie/babbage/ada)对照实验",
    openDVCFTName: CONST_DAVINCI_FINE_TUNNING,
    openDVCFTNameRp2: CONST_DAVINCI_FINE_TUNNING_REPEAT2,
    openDVCFTNameRp4: CONST_DAVINCI_FINE_TUNNING_REPEAT4,
    openCURIEFTName: CONST_CURIE_FINE_TUNNING,
    openCURIEFTNameRp2: CONST_CURIE_FINE_TUNNING_REPEAT2,
    openCURIEFTNameRp4: CONST_CURIE_FINE_TUNNING_REPEAT4,
    openBABBAGEFTName: CONST_BABBAGE_FINE_TUNNING,
    openBABBAGEFTNameRp2: CONST_BABBAGE_FINE_TUNNING_REPEAT2,
    openBABBAGEFTNameRp4: CONST_BABBAGE_FINE_TUNNING_REPEAT4,
    openADAFTName: CONST_ADA_FINE_TUNNING,
    openADAFTNameRp2: CONST_ADA_FINE_TUNNING_REPEAT2,
    openADAFTNameRp4: CONST_ADA_FINE_TUNNING_REPEAT4,
  });
})

textComplRouter.get('/demo3', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index3.pug', {
    // title: "单句问答(OpenAi-davinci)对照实验 - 可分配user_info",
    ftNameWithUserInfo: CONST_DAVINCI_USERINFO_FINE_TUNNING,
    ftNameWithUserInfo_RP2: CONST_DAVINCI_USERINFO_FINE_TUNNING_RP2
  })
})

textComplRouter.post('/completionByModelByComKey', async (ctx: {
  request: {
    body: {
      ques: string,
      user: string,
      model: string,
    }
  },
  body: ICommonResult<string>
}, next: Function) => {
  console.log('completionByModel start:')
  let ques = ''
  let user = ''
  let model = ''
  console.log("ctx.request.body=> type:", typeof ctx.request.body, JSON.stringify(ctx.request.body))
  if (typeof ctx.request.body === "string") {
    let json = JSON.parse(ctx.request.body)
    ques = json.ques
    user = json.user
    model = json.model
  }
  else {
    ques = ctx.request.body.ques
    user = ctx.request.body.user
    model = ctx.request.body.model
  }

  let resp: Response
  const completionParams = JSON.parse(JSON.stringify(CONST_DEFAULT_COMPLETION_PARAM)) as CompletionParams
  completionParams.user = user
  completionParams.model = model
  completionParams.prompt = ques
  completionParams.max_tokens = 2000
  console.log('使用公司key...', {
    method: "post",
    headers: {
      'Authorization': 'Bearer ' + CONST_OPENAI_COM_KEY,
      'Content-Type': 'application/json'
    },
    body: JSON.stringify(completionParams)
  })
  try {
    resp = await fetch('https://api.openai.com/v1/completions', {
      method: "post",
      headers: {
        'Authorization': 'Bearer ' + CONST_OPENAI_COM_KEY,
        'Content-Type': 'application/json'
      },
      body: JSON.stringify(completionParams)
    })
  } catch (ex: any) {
    ctx.body = {
      Result: 0,
      code: 10510,
      Reason: "fetch err:" + ex.message
    };
    return
  }

  const json = await resp.json()
  if (json && json.error) {
    ctx.body = {
      Result: 0,
      code: 500,
      data: json
    }
    return
  }
  ctx.body = {
    Result: 1,
    code: 200,
    data: json
  }
})

textComplRouter.post('/completionByModel', async (ctx: {
  request: {
    body: {
      ques: string,
      user: string,
      model: string,
    }
  },
  body: ICommonResult<string>
}, next: Function) => {
  console.log('completionByModel start:')
  let ques = ''
  let user = ''
  let model = ''
  console.log("ctx.request.body=> type:", typeof ctx.request.body, JSON.stringify(ctx.request.body))
  if (typeof ctx.request.body === "string") {
    let json = JSON.parse(ctx.request.body)
    ques = json.ques
    user = json.user
    model = json.model
  }
  else {
    ques = ctx.request.body.ques
    user = ctx.request.body.user
    model = ctx.request.body.model
  }

  let resp: Response
  const completionParams = JSON.parse(JSON.stringify(CONST_DEFAULT_COMPLETION_PARAM)) as CompletionParams
  completionParams.user = user
  completionParams.model = model
  completionParams.prompt = ques
  try {
    resp = await fetch('https://api.openai.com/v1/completions', {
      method: "post",
      headers: {
        'Authorization': 'Bearer ' + CONST_OPENAI_KEY,
        'Content-Type': 'application/json'
      },
      body: JSON.stringify(completionParams)
    })
  } catch (ex: any) {
    ctx.body = {
      Result: 0,
      code: 10510,
      Reason: "fetch err:" + ex.message
    };
    return
  }

  const json = await resp.json()
  if (json && json.error) {
    ctx.body = {
      Result: 0,
      code: 500,
      data: json
    }
    return
  }
  ctx.body = {
    Result: 1,
    code: 200,
    data: json
  }
})

textComplRouter.post('/completion', async (ctx: {
  request: {
    body: {
      ques: string,
      user: string
    }
  },
  body: ICommonResult<string>
}, next: Function) => {
  console.log('completion start:')
  let ques = ''
  let user = ''
  console.log("ctx.request.body=> type:", typeof ctx.request.body, JSON.stringify(ctx.request.body))
  if (typeof ctx.request.body === "string") {
    let json = JSON.parse(ctx.request.body)
    ques = json.ques
    user = json.user
  }
  else {
    ques = ctx.request.body.ques
    user = ctx.request.body.user
  }

  let resp: any
  const completionParams = JSON.parse(JSON.stringify(CONST_DEFAULT_COMPLETION_PARAM)) as CompletionParams
  completionParams.user = user
  completionParams.prompt = ques
  completionParams.model = CONST_DAVINCI_FINE_TUNNING
  console.log('step 1.', JSON.stringify(completionParams))
  try {
    resp = await fetch('https://api.openai.com/v1/completions', {
      method: "post",
      headers: {
        'Authorization': 'Bearer ' + CONST_OPENAI_KEY,
        'Content-Type': 'application/json'
      },
      body: JSON.stringify(completionParams)
    }).then(json => json.json())
    console.log('step 2.')
  } catch (ex: any) {
    ctx.body = {
      Result: 0,
      code: 10510,
      Reason: "fetch err:" + ex.message
    };
    return
  }
  console.log('step 3.')
  if (resp && resp.error) {
    ctx.body = {
      Result: 0,
      code: 500,
      data: resp
    }
    return
  }
  const choices = resp.choices as { text: string }[]
  for (var i = 0; i < choices.length; i++) {
    // console.log('00000000000', choices[i].text);
    choices[i].text = choices[i].text.split(CONST_COLPLETION_END)[0]
  }
  ctx.body = {
    Result: 1,
    code: 200,
    data: resp
  }
})
export default textComplRouter
