import fetch, { Response } from 'node-fetch';
import { CONST_OPENAI_COM_KEY, CONST_WISPER_V2T_MODEL } from "../config";
import { ICommonResult } from "../utils/http";
import FormData = require('form-data');
import fs = require('fs');
import path = require('path');

const Router = require('koa-router'); // 引入koa-router
const wisperTransRouter = new Router()

wisperTransRouter.get('/demomb', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index-v2t-mb.pug', {
    // title: "单句问答(OpenAi-davinci)对照实验",
    openV2TName: CONST_WISPER_V2T_MODEL,
    // openFTNameRp4: CONST_DAVINCI_FINE_TUNNING_REPEAT4,
  });
})
wisperTransRouter.get('/demomb2', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index-v2t-mb2.pug', {
    // title: "单句问答(OpenAi-davinci)对照实验",
    openV2TName: CONST_WISPER_V2T_MODEL,
    // openFTNameRp4: CONST_DAVINCI_FINE_TUNNING_REPEAT4,
  });
})

wisperTransRouter.get('/demopc', async (ctx: any, next: Function) => {
  ctx.type = 'html';
  ctx.body = await ctx.render('index-v2t-pc.pug', {
    // title: "单句问答(OpenAi-davinci)对照实验",
    openV2TName: CONST_WISPER_V2T_MODEL,
    // openFTNameRp4: CONST_DAVINCI_FINE_TUNNING_REPEAT4,
  });
})

async function wisperRequest(url: string, ctx: {
  // uploadpath: string,
  request: {
    files: {
      file: {
        // size: number,
        filepath: string,
        // path: any,
        // newFilename: string,
        // mimetype: "audio/mpeg",
        // mtime: string,
        // originalFilename: string
      }
    },
    body: {
      userId: string
    }
  }
}): Promise<ICommonResult<string>> {
  // await new Promise((resolve) => {
  //   setTimeout(() => {
  //     resolve({})
  //   }, 1000)
  // })

  // if (process.env.NODE_ENV === 'development') {
  //   return {
  //     Result: 1,
  //     code: 200,
  //     data: "Do you know who is the American president?"
  //   };
  // }

  const file = ctx.request.files.file
  // const newFileName = ctx.request.files.file.newFilename + '_' + ctx.request.files.file.originalFilename
  // const newFilePath = path.resolve(__dirname, '/uploads/' + newFileName)

  let resp: Response
  let formData = new FormData()
  const fileStream = fs.createReadStream(file.filepath)
  formData.append('file', fileStream)
  formData.append('model', CONST_WISPER_V2T_MODEL)
  const headers = formData.getHeaders({ 'Authorization': 'Bearer ' + CONST_OPENAI_COM_KEY })

  console.log(file.filepath, JSON.stringify({
    method: "post",
    headers,
    body: formData
  }))
  try {
    resp = await fetch(url, {
      method: "POST",
      headers,
      body: formData
    })
  } catch (ex: any) {
    return {
      Result: 0,
      code: 10510,
      Reason: "fetch err:" + ex.message
    };
  }

  const txt = await resp.text()
  console.log("obj=>", txt)
  return {
    Result: 1,
    code: 200,
    data: txt
  }
}

wisperTransRouter.post('/transcription', async (ctx: {
  // uploadpath: string,
  request: {
    files: {
      file: {
        // size: number,
        filepath: string,
        // path: any,
        // newFilename: string,
        // mimetype: "audio/mpeg",
        // mtime: string,
        // originalFilename: string
      }
    },
    body: {
      userId: string
    }
  },
  body: ICommonResult<string>
}, next: Function) => {
  console.log('wisperTransRouter.transcription start:', typeof process.env.NODE_ENV, '[' + process.env.NODE_ENV + ']')
  ctx.body = await wisperRequest('https://api.openai.com/v1/audio/transcriptions', ctx)
})

wisperTransRouter.post('/translation', async (ctx: {
  request: {
    files: {
      file: {
        // size: number,
        filepath: string,
        // path: any,
        // newFilename: string,
        // mimetype: "audio/mpeg",
        // mtime: string,
        // originalFilename: string
      }
    },
    body: {
      userId: string
    }
  },
  body: ICommonResult<any>
}, next: Function) => {
  console.log('wisperTransRouter.translation start:', typeof process.env.NODE_ENV, '[' + process.env.NODE_ENV + ']')
  ctx.body = await wisperRequest('https://api.openai.com/v1/audio/translations', ctx)
})

export default wisperTransRouter
