export interface CompletionParams {
  /**
   * Required
   * ID of the model to use. You can use the List models API to see all of your available models, or see our Model overview for descriptions of them.
   * 要使用的模型的 ID。可以使用列表模型 API 查看所有可用模型，或参阅模型概述了解它们的描述。
   * */
  model: string

  /**
   * Optional
   * Defaults to <|endoftext|>
   * The prompt(s) to generate completions for, encoded as a string, array of strings, array of tokens, or array of token arrays.
   * Note that <|endoftext|> is the document separator that the model sees during training, so if a prompt is not specified the model will generate as if from the beginning of a new document.
   * 用于生成完成、编码为字符串、字符串数组、标记数组或标记数组数组的提示。
   * 请注意，<|endoftext|> 是模型在训练期间看到的文档分隔符，因此如果未指定提示，模型将生成，就像从新文档的开头一样。
   * */
  prompt?: string | string[]

  /**
   * Optional
   * Defaults to null
   * The suffix that comes after a completion of inserted text.
   * 完成插入文本后的后缀。
   */
  suffix?: string

  /**
   * Optional
   * Defaults to 16
   * The maximum number of tokens to generate in the completion.
   * The token count of your prompt plus max_tokens cannot exceed the model's context length. Most models have a context length of 2048 tokens (except for the newest models, which support 4096).
   * 完成时要生成的最大令牌数。
   * 提示加max_tokens的令牌计数不能超过模型的上下文长度。大多数模型的上下文长度为 2048 个令牌（最新模型除外，它支持 4096）。
   */
  max_tokens?: number


  /**
   * Optional
   * Defaults to 1
   * What sampling temperature to use, between 0 and 2. Higher values like 0.8 will make the output more random, while lower values like 0.2 will make it more focused and deterministic.
   * We generally recommend altering this or top_p but not both.
   * 使用什么采样温度，介于 0 和 2 之间。较高的值（如 0.8）将使输出更加随机，而较低的值（如 0.2）将使其更加集中和确定。
   * 我们通常建议更改此或top_p，但不要同时更改两者。
   */
  temperature?: number

  /**
   * Optional
   * Defaults to 1
   * An alternative to sampling with temperature, called nucleus sampling, where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens comprising the top 10% probability mass are considered.
   * We generally recommend altering this or temperature but not both.
   * 使用温度采样的替代方法称为核心采样，其中模型考虑具有top_p概率质量的令牌的结果。因此，0.1 意味着只考虑包含前 10% 概率质量的token。
   * 我们通常建议更改此温度或温度，但不要同时更改两者。
   */
  top_p?: number

  /**
   * Optional
   * Defaults to 1
   * How many completions to generate for each prompt.
   * Note: Because this parameter generates many completions, it can quickly consume your token quota. Use carefully and ensure that you have reasonable settings for max_tokens and stop.
   * 为每个提示生成的完成次数。
   * 注意：由于此参数会生成许多完成，因此它会快速消耗您的令牌配额。谨慎使用，并确保对max_tokens和停止设置合理。
   */
  n?: number

  /**
   * Optional
   * Defaults to false
   * Whether to stream back partial progress. If set, tokens will be sent as data-only server-sent events as they become available, with the stream terminated by a data: [DONE] message.
   * 是否流式传输回部分进度。如果设置，令牌将在可用时作为纯数据服务器发送的事件发送，流由 data： [DONE] 消息终止。
   */
  stream?: boolean

  /**
   * Optional
   * Defaults to null
   * Include the log probabilities on the logprobs most likely tokens, as well the chosen tokens. For example, if logprobs is 5, the API will return a list of the 5 most likely tokens. The API will always return the logprob of the sampled token, so there may be up to logprobs+1 elements in the response.
   * The maximum value for logprobs is 5. If you need more than this, please contact us through our Help center and describe your use case.
   * 包括 logprobs 最可能的令牌上的对数概率，以及所选令牌。例如，如果 logprobs 为 5，则 API 将返回 5 个最可能的令牌的列表。API 将始终返回采样令牌的 logprob，因此响应中最多可能有 logprobs+1 个元素。
   * 对数的最大值为 5。如果您需要更多，请通过我们的帮助中心与我们联系并描述您的使用案例。
   */
  logprobs?: number

  /**
   * Optional
   * Defaults to false
   * Echo back the prompt in addition to the completion
   * 除了完成之外，还回显提示
   */
  echo?: boolean

  /**
   * Optional
   * Defaults to null
   * Up to 4 sequences where the API will stop generating further tokens. The returned text will not contain the stop sequence.
   * 最多 4 个序列，其中 API 将停止生成更多令牌。返回的文本将不包含停止序列。
   */
  stop?: string | string[]

  /**
   * Optional
   * Defaults to 0
   * Number between -2.0 and 2.0. Positive values penalize new tokens based on whether they appear in the text so far, increasing the model's likelihood to talk about new topics.
   * See more information about frequency and presence penalties.
   * 介于 -2.0 和 2.0 之间的数字。正值会根据新标记到目前为止是否出现在文本中来惩罚它们，从而增加模型讨论新主题的可能性。
   * 查看有关频率和存在处罚的更多信息。
   */
  presence_penalty?: number

  /**
   * Optional
   * Defaults to 0
   * Number between -2.0 and 2.0. Positive values penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line verbatim.
   * See more information about frequency and presence penalties.
   * 介于 -2.0 和 2.0 之间的数字。正值会根据新标记到目前为止在文本中的现有频率来惩罚新标记，从而降低模型逐字重复同一行的可能性。
   * 查看有关频率和存在处罚的更多信息。
   */
  frequency_penalty?: number

  /**
   * Optional
   * Defaults to 1
   * Generates best_of completions server-side and returns the "best" (the one with the highest log probability per token). Results cannot be streamed.
   * When used with n, best_of controls the number of candidate completions and n specifies how many to return – best_of must be greater than n.
   * Note: Because this parameter generates many completions, it can quickly consume your token quota. Use carefully and ensure that you have reasonable settings for max_tokens and stop.
   * 在服务器端生成best_of完成并返回“最佳”（每个令牌的对数概率最高的完成）。无法流式传输结果。
   * 当与 n 一起使用时，best_of 控制候选完成的数量，n 指定要返回的数量 - best_of必须大于 n。
   * 注意：由于此参数会生成许多完成，因此它会快速消耗您的令牌配额。谨慎使用，并确保对max_tokens和停止设置合理。
   */
  best_of?: number

  /**
   * Optional
   * Defaults to null
   * Modify the likelihood of specified tokens appearing in the completion.
   * Accepts a json object that maps tokens (specified by their token ID in the GPT tokenizer) to an associated bias value from -100 to 100. You can use this tokenizer tool (which works for both GPT-2 and GPT-3) to convert text to token IDs. Mathematically, the bias is added to the logits generated by the model prior to sampling. The exact effect will vary per model, but values between -1 and 1 should decrease or increase likelihood of selection; values like -100 or 100 should result in a ban or exclusive selection of the relevant token.
   * As an example, you can pass {"50256": -100} to prevent the <|endoftext|> token from being generated.
   * 修改完成中出现指定令牌的可能性。
   * 接受将令牌（由其在 GPT 标记器中的令牌 ID 指定）映射到 -100 到 100 之间的关联偏差值的 json 对象。您可以使用此分词器工具（适用于 GPT-2 和 GPT-3）将文本转换为令牌 ID。在数学上，偏差在采样之前被添加到模型生成的对数中。确切的效果因模型而异，但介于 -1 和 1 之间的值应降低或增加选择的可能性;像 -100 或 100 这样的值应该会导致禁止或排他性选择相关令牌。
   * 例如，您可以传递 {“50256”： -100} 以防止生成 <|endoftext|> 令牌。
   */
  logit_bias?: Map<string, number>

  /**
   * Optional
   * A unique identifier representing your end-user, which can help OpenAI to monitor and detect abuse. Learn more.
   * 代表最终用户的唯一标识符，可帮助 OpenAI 监控和检测滥用行为。了解更多信息。
   */
  user?: string
}
