import log4js, { Appender, Configuration } from 'koa-log4'
import path from 'path'
// appenders: { [name: string]: Appender; };
// categories: { [name: string]: { appenders: string[]; level: string; } };
// pm2?: boolean;
// pm2InstanceVar?: string;
// levels?: Levels;
// disableClustering?: boolean;
log4js.configure({
  appenders: {
    error: {
      type: 'file',           //日志类型
      category: 'errLogger',    //日志名称
      filename: path.join('logs/', 'error/error.log'), //日志输出位置，当目录文件或文件夹不存在时自动创建
      maxLogSize: 104800, // 文件最大存储空间
      backups: 100  //当文件内容超过文件存储空间时，备份文件的数量
    } as Appender,
    response: {
      type: 'dateFile',
      category: 'resLogger',
      filename: path.join('logs/', 'responses/'),
      pattern: 'yyyy-MM-dd.log', //日志输出模式
      alwaysIncludePattern: true,
      maxLogSize: 104800,
      backups: 100
    } as Appender
  },
  categories: {
    error: { appenders: ['error'], level: 'info' },
    response: { appenders: ['response'], level: 'info' },
    default: { appenders: ['response'], level: 'info' }
  },
  // pm2?: boolean;
  // pm2InstanceVar?: string;
  // levels?: Levels;
  // disableClustering?: boolean;
  // replaceConsole: true
} as Configuration)


let logger: {
  errLogger: Function,
  resLogger: Function
} = {
  errLogger: () => { },
  resLogger: () => { }
}

const formatError = (ctx: any, err: any) => {
  const { method, url } = ctx
  let body = ctx.request.body
  // const user = ctx.state.user
  return { method, url, body, /*user,*/ err }
}

const formatRes = (ctx: any, costTime: number) => {
  const { method, url, response: { status, message, body }, request: { header: { token } } } = ctx
  let Result = 'no init'
  if (body) {
    Result = body.Result
  }
  let reqBody = ctx.request.body
  // const user = ctx.state.user
  return { method, url, /*user,*/ body: reqBody, costTime, token, response: { status, message, body: { Result } } }
}


let errorLogger = log4js.getLogger('error')
let resLogger = log4js.getLogger('response')

// 封装错误日志
logger.errLogger = (ctx: any, error: any) => {
  if (ctx && error) {
    errorLogger.error(formatError(ctx, error))
  }
}

// 封装响应日志
logger.resLogger = (ctx: any, resTime: any) => {
  if (ctx) {
    resLogger.info(formatRes(ctx, resTime))
  }
}

export default logger
